/******************************* Copyright notice *******************************
Copyright (C)2002 by Andrei Loskutov <loskutov@gmx.de>. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
********************************************************************************/

package adiag.connection.command.eclipse;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareEditorInput;
import org.eclipse.compare.ITypedElement;
import org.eclipse.compare.structuremergeviewer.Differencer;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubProgressMonitor;

/**
  */
public class EclipseCompare extends CompareEditorInput {

    
    /** Stores reference to the element displayed on the left side of the viewer. */
    private ITypedElement left;
    /** Stores reference to the element displayed on the right side of the viewer. */
    private ITypedElement right;

    /** Constructor for PerforceCompareEditorInput.
      * 
      * @param left element displayed on the left.
      * @param right element displayed on the right.
      * @param ancestor element displayed in ancestor pane.
      */
    public EclipseCompare(
        ITypedElement left,
        ITypedElement right) {
        super(new CompareConfiguration());
        this.left = left;
        this.right = right;
    }

    /** @see CompareEditorInput#prepareInput(IProgressMonitor) */
    protected Object prepareInput(IProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException {
        if (right == null || left == null) {
            return null;
        }        
                
        try {
            initLabels();
            Differencer differencer = new Differencer();
            monitor.beginTask("BranchView: comparing...", 30);
            IProgressMonitor sub = new SubProgressMonitor(monitor, 10);
            try {
                sub.beginTask("BranchView: comparing...", 100);
                
                return differencer.findDifferences(
                    false,
                    sub,
                    null,
                    null,
                    left,
                    right);
            } finally {
                sub.done();
            }
        } catch (OperationCanceledException e) {
            throw new InterruptedException(e.getMessage());
        } finally {
            monitor.done();
        }
    }

    /**
      * Sets up the title and pane labels for the comparison view.
      */
    private void initLabels() {
        CompareConfiguration cc =
            (CompareConfiguration) getCompareConfiguration();

        cc.setLeftLabel(left.getName());
        cc.setLeftImage(left.getImage());

        cc.setRightLabel(right.getName());
        cc.setRightImage(right.getImage());

        setTitle("BranchView: compare");
    }


}
